package duc.util.gui;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import duc.util.*;

/**
 * TelexDriver
 * @version	0.2
 * @author	Ho Ngoc Duc <duc@informatik.uni-leipzig.de>
 */
public abstract class AbstractVietKey extends KeyAdapter {
    protected Encoding encoding;
    protected boolean enabled = true;
    private BitSet MODIFIERS;
	String fToneMarks;
	String fInputMethod;
	char fBreve, fHorn, fStroke;
	String fCirc;
/**
 * 
 */
public AbstractVietKey(String method) {

	fInputMethod = method;
	if ("VNI".equalsIgnoreCase(method)) {
		setVNIInput();
	} else if ("VIQR".equalsIgnoreCase(method)) {
		setVIQRInput();
	} else {
		setTelexInput();
	}

    MODIFIERS = new BitSet(128);
    for (int i = 0; i < 128; i++) {
        if (fCirc.indexOf((char) i) >= 0)
            MODIFIERS.set(i);
        else if (isBreve((char) i, 'a'))
            MODIFIERS.set(i);
        else if (isHorn((char) i, 'o'))
            MODIFIERS.set(i);
        else if (isStroke((char) i, 'd'))
            MODIFIERS.set(i);
        else if (isToneMark((char) i))
            MODIFIERS.set(i);
    }
}
	protected char charAt(int i) { return getText().charAt(i); }
	private static boolean eq(char c1, char c2) {
	return Character.toLowerCase(c1) == c2;
	}
	protected abstract int getCaretPosition();
	protected abstract String getText();
   public int getToneMarkId(char c) {return fToneMarks.indexOf(c);}
   protected int indexOfLastVowel(int pos)
   {
   	int beg = Math.max(0,pos-3);
   	for (int i=pos-1; i>=beg; i--)
   	{
	    char c = charAt(i);
	    if (encoding.isVowel(c)) return i;
   	}
   	return -1;
   }
	protected int indexOfToneCarrier(int pos)
	{
   	int idx = indexOfLastVowel(pos);
   	if (idx <= 0) return idx;
   	char c = charAt(idx-1);
   	if (!encoding.isVowel(c) && !eq(c,'q')) return idx;
   	if (encoding.hasDiacritic(charAt(idx))) return idx;
   	if (encoding.hasDiacritic(c)) return idx -1;
   	if (eq(c,'q') && eq(charAt(idx),'u')) return -1;
   	if (eq(c,'o') && eq(charAt(idx),'a')) return idx;
   	if (eq(c,'o') && eq(charAt(idx),'e')) return idx;
   	if (eq(c,'u') && eq(charAt(idx),'y')) return idx;
   	if ((idx >= 2) && eq(c,'u') && eq(charAt(idx-2),'q')) return idx;
   	if ((idx >= 2) && eq(c,'i') && eq(charAt(idx-2),'g')) return idx;
   	return idx -1;
	}
	   public boolean isBreve(char c, char ch) {
	   return eq(c, fBreve);
	   }
public boolean isCircumflex(char c, char ch) {
    if (fCirc.indexOf(c) < 0)
        return false;
    if ("Telex".equalsIgnoreCase(fInputMethod))
        return eq(c, Character.toLowerCase(ch));
    return true;
}
   public boolean isHorn(char c, char ch) {return eq(c, fHorn);}
   protected boolean isModifier(char c)
   {
	  return MODIFIERS.get((int)c);
   }
   public boolean isStroke(char c, char ch) {return eq(c, fStroke);}
   // The 6 accents: empty, grave, hook above, tilde, acute, dot below
   public boolean isToneMark(char c) {return getToneMarkId(c) >= 0;}
public void keyPressed(KeyEvent e) {
	switch (e.getKeyCode()) {
		case KeyEvent.VK_F1 :
			{
			    toggleMode();
			    e.consume();
			    return;
			}
	}
}
   protected void modifyText(KeyEvent e)
   {
	   if (!enabled) return;
	  char c = e.getKeyChar();
	  if(!isModifier(c))
		 return;
	  int pos = getCaretPosition();
	  if(pos <= 0)
		 return;
	  int idx = pos-1; // position of the character to be modified
	  char last = charAt(idx);
	  char newVal = last;
	  if (isCircumflex(c, last))
	   	newVal = encoding.addCircumflex(last);
	  else if (isBreve(c, last))
	  newVal = encoding.addBreveHorn(last);
	  else if (isHorn(c, last))
	  newVal = encoding.addBreveHorn(last);
	  else if (isStroke(c, last))
	  newVal = encoding.addStroke(last);
	  else if (isToneMark(c))
	  {
	  	idx = indexOfToneCarrier(pos);
	if (idx < 0) return;
	  	last = charAt(idx);
	newVal = encoding.modifyTone(last, getToneMarkId(c));
	  }
	  if (last != newVal)
	  {
	  replaceCharacter(idx, newVal);
	  setCaretPosition(pos);
	  e.consume();
	  }
   }
	protected abstract void replaceCharacter(int idx, char c);
	protected abstract void setCaretPosition(int pos);
void setTelexInput() {
   fToneMarks = "zfrxsj";
   fHorn = 'w';
   fStroke = 'd';
   fBreve = 'w';
   fCirc = "AaEeOo";
}
	protected abstract void setText(String str);
void setVIQRInput() {
   fToneMarks = "z`?~'.";
   fHorn = '+';
   fStroke = 'd';
   fBreve = '(';
   fCirc = "^";
}
void setVNIInput() {
   fToneMarks = "023415";
   fHorn = '7';
   fStroke = '9';
   fBreve = '8';
   fCirc = "6";
}
	public void toggleMode() { enabled = !enabled; }
}
